/*
	This application illustrates inheritance, polymorphism,
	and abstract classes.
	
	Custom classes: ShapeApp
				Shape				a common abstract ancestor
                Rectangle           defines and draws rectangles
                Ellipse             defines and draws ellipses
                Square              defines and draws squares

*/

import java.awt.*;

public class ShapeApp {

	public static void main(String args[]) {
		Shape a, b, c, d;

		printHead();
		
		System.out.println( "Set 'a' to a new default rectangle." );
		a = new myRectangle(10,10);
		outputStuff( a );

		System.out.println( "Set 'b' to a new rectangle." );
		b = new myRectangle(20, 20, 15, 30, Color.blue);
		outputStuff( b );

		System.out.println( "Move 'b' to (75,25)." );
		b.moveTo(75, 25);
		outputStuff( b );

		System.out.println( "Change 'a' to the same size as 'b'." );
		a.wide = b.wide;
		a.high = b.high;
		outputStuff( a );
		
		System.out.println( "Set 'c' to a new default ellipse." );
		c = new myEllipse(30, 30);
		outputStuff( c );
		
		System.out.println( "Reset 'a' to a new ellipse." );
		a = new myEllipse(50, 50, 100, 50, Color.green);
		outputStuff( a );
		
		System.out.println( "Set 'd' to a new default square." );
		d = new mySquare(40, 40);
		outputStuff( d );
		
		System.out.println( "Do a bad thing - stretch the square, 'd'." );
		d.wide = 2*d.wide;
		outputStuff( d );
		
		printTail();
		
	}
	
	public static void outputStuff( Shape x ) {
		System.out.println( x );
		System.out.println( "Area: " + x.area());
		System.out.println( "Perimeter: " + x.perimeter());
		System.out.println( );
	}
	
	public static void printHead() {
		System.out.println( "Content-type: text/html" );
		System.out.println( "" );
		System.out.println( "<html>" );
		System.out.println( "<head>" );
		System.out.println( "<title>ShapeApp Example</title>" );
		System.out.println( "</head>" );
		System.out.println( "<body bgcolor=\"#FFFFFF\" alink=\"#0080FF\">" );
		System.out.println( "<h1>Shape Application</h1>" );
		System.out.println( "<blockquote><pre><tt>" );
		
	}
	
	public static void printTail() {
		System.out.println( "</tt></pre></blockquote>" );
		System.out.println( "<hr> <a href=\"ShapeApp.java\">The source.</a>");
		System.out.println( "</body>" );
		System.out.println( "</html>" );
	}

}

/** 
	The Shape class is an abstract class which defines the fields, methods
    and behavior common to all shapes.
*/

abstract class Shape {
	static public final int defaultWidth = 20, defaultHeight = 10;
	static public final Color defaultColor = Color.red;
   
	Color color;
	int x;
	int y;
	int wide;
	int high;

	Shape(int x, int y, int w, int h, Color c) {
		this.x = x;
		this.y = y;
		this.wide = w;
		this.high = h;
		this.color = c;
	}
	
	void moveTo(int x, int y) {
		this.x = x;
		this.y = y;
	}
	
	abstract void draw(Graphics g);

	abstract float area();

	abstract float perimeter();
	
	public String toString() {
		String myInfo;
		myInfo = "at (" + x + "," + y + "), ";
		myInfo += wide + " pixels wide, " + high + " pixels high.\n";
		myInfo += color;
		return( myInfo );
	}
}


/** The concrete Rectangle class. */

class myRectangle extends Shape{
	
	myRectangle(int x, int y, int w, int h, Color c) {
		super(x, y, w, h, c);
	}
	
	myRectangle(int x, int y) {
		super(x, y, defaultWidth, defaultHeight, defaultColor);
	}
	
	void draw(Graphics g) {
		g.setColor(this.color);
		g.fillRect(this.x, this.y, this.wide, this.high);  
	}

	public float area() {
		return( wide*high );
	}
	
	public float perimeter() {
		return( 2*(wide + high) );
	}

	public String toString() {
		return( "Rectangle:  " + super.toString() );
	}
}

/** The concrete Ellipse class. */

class myEllipse extends Shape{
	
	myEllipse(int x, int y, int w, int h, Color c) {
		super(x, y, w, h, c);
	}
	
	myEllipse(int x, int y) {
		this(x, y, defaultWidth, defaultHeight, defaultColor);
	}
	
	void draw(Graphics g) {
		g.setColor(this.color);
		g.fillOval(this.x, this.y, this.wide, this.high);  
	}

	public float area() {
		return( (float)(Math.PI*0.25*wide*high) );
	}
	
	public float perimeter() {
		return( (float)(Math.PI*0.5*(wide + high)) );
	}

	public String toString() {
		return( "Ellipse:  " + super.toString() );
	}
}

/** The concrete Square class. */

class mySquare extends myRectangle{
	
	mySquare(int x, int y, int w, Color c) {
		super(x, y, w, w, c);
	}
	
	mySquare(int x, int y) {
		this(x, y, defaultWidth, defaultColor);
	}
	
	public String toString() {
		String myInfo = super.toString();
		int myLength = myInfo.length();
		return( "Square:  " + myInfo.substring(11, myLength) );
	}
}

