/* -------------------------------------------------------------
This applet paints a rectangle, ellipse or square of the color you've chosen
wherever you click. This applet keeps a list of the shapes you've drawn
and paints all the shapes in the list when it repaints.

Java's classes: Applet    (applet)
                Event     (awt)     user-generated action
                Graphics  (awt)     used for drawing
                Color     (awt)     defines colors
                Choice    (awt)     shape and color selection choices
                Vector    (util)    list of shapes

Custom classes: SimpleDraw
				Shape				a common ancestor for circles and squares
                Rectangle           defines and draws rectangles
                Ellipse             defines and draws ellipses
                Square              defines and draws squares

------------------------------------------------------------- */

import java.applet.Applet;
import java.util.*;
import java.awt.*;
import ShapePack.*;

public class SimpleDraw extends Applet {
	public static final int defaultWidth = 150, defaultHeight = 75;
	
   Vector  drawnShapes;
   Choice  shapeChoice;
   Choice  colorChoice;
   Button  clearButton;
   
   /** Create the GUI. */
   public void init() {
      drawnShapes = new Vector();
      
      shapeChoice = new Choice();
      shapeChoice.addItem("Rectangle");
      shapeChoice.addItem("Ellipse");
      shapeChoice.addItem("Square");
      add(shapeChoice);
      
      colorChoice = new Choice();
      colorChoice.addItem("Red");
      colorChoice.addItem("Green");
      colorChoice.addItem("Blue");
      add(colorChoice);
      
      clearButton = new Button("Clear");
      clearButton.setBackground(Color.white);
      add(clearButton);
   }
   
   /** Draw all the shapes. */
   public void paint(Graphics g) {
      Shape s;
      int numShapes;
      
      numShapes = drawnShapes.size();
      for (int i = 0; i < numShapes; i++) {
      
         s = (Shape)drawnShapes.elementAt(i);
         
         // When the shape draws, circles and squares each invoke their own
         // draw method, depending on which shape this is.
         s.draw(g);  
      }
   }
   
   /** Create a new shape. */
   public boolean mouseUp(Event e, int x, int y) {
   
      Shape s;  // This shape will be either a circle or a square.
      Color c;
      
      System.out.println("MouseUp at (" + x + ", " + y + ")");

      String shapeString = shapeChoice.getSelectedItem();
      String colorString = colorChoice.getSelectedItem();
      
      if (colorString.equals("Red"))
         c = Color.red;
      else if (colorString.equals("Green"))
         c = Color.green;
      else
         c = Color.blue;
      
      if (shapeString.equals("Rectangle"))
         s = new myRectangle(x, y, defaultWidth, defaultHeight, c);
      else if (shapeString.equals("Ellipse"))
         s = new myEllipse(x, y, defaultWidth, defaultHeight, c);
      else
         s = new mySquare(x, y, defaultHeight, c);
      
      outputStuff( s );
      drawnShapes.addElement(s);

      repaint();
      
      return true;
   }

   /** Create a new shape. */
   public boolean action(Event e, Object arg) {
   
      
      System.out.println("Action Event");
	  if (e.target == clearButton) {
      	 drawnShapes.removeAllElements();
	  }
      repaint();
      
      return true;
   }
	
	public static void outputStuff( Shape x ) {
		System.out.println( x );
		System.out.println( "Area: " + x.area());
		System.out.println( "Perimeter: " + x.perimeter());
		System.out.println( );
	}

}
