import java.util.*;
 
public class BalkingQueue extends M865Queue {
	protected LinkedList list;
	
	public BalkingQueue() {
		list = new LinkedList();
	}
	
	protected BalkingQueue(long hash, LinkedList list) {
		this.hash = hash;
		this.list = list;
	}
	
	
	public void enqueue(Object obj) {
	// pushes an object onto the end of the queue.
		updateHashCode(obj);
		list.addLast(obj);
	}
	
	public Object dequeue() {
	// removes and returns the object at the front of the queue.
	// returns null if the queue is empty.
		if (list.size() == 0) {
			return null;
		} else {
			Object obj = list.removeFirst();
			downdateHashCode(obj);
			return obj;
		}
	}
	
	public Object peek() {
	// returns the object at the front of the queue.
	// returns null if the queue is empty.
		if (list.size() == 0) {
			return null;
		} else {
			return list.getFirst();
		}
	}

	public void clear() {
	// Removes all of the elements from this collection (optional operation). 
		list.clear();
		hash = 0;
	}
		
	public Object clone() {
	// Returns a copy of this queue
		return (new BalkingQueue(hash, (LinkedList)list.clone()));
	}
		
	public class BalkingQueueIterator implements Iterator {
		private BalkingQueue myQ;
		private Iterator myIter;
		private long check;
		private Object last;
		
		BalkingQueueIterator(BalkingQueue q) {
			myQ = q;
			check = myQ.hash;
			myIter = myQ.list.iterator();
			last = null;
		}

		public boolean hasNext() {
			//  Check that the queue has not been altered during the processing
			//  of this iterator
			if (check != myQ.hash) {
				throw( new ConcurrentModificationException("This BalkingQueue has been altered"));
			} else {
				return (myIter.hasNext());
			}
		}
		
		public Object next() {
			if (check != myQ.hash) {
				throw( new ConcurrentModificationException("This BalkingQueue has been altered"));
			} else if (myIter.hasNext()) {
				last = myIter.next();
				return last;
			} else {
				return null;
			}
		}
		
		public void remove() {
		// Removes the last element returned by the iterator.
			if (check != myQ.hash) {
				throw( new ConcurrentModificationException("This BalkingQueue has been incorrectly altered"));
			} else if (last == null) {
				throw( new UnsupportedOperationException("Can only remove() the object returned by next()"));
			} else {
				myQ.downdateHashCode(last);
				check = myQ.hash;
				myIter.remove();
			}
		}
	}

	public Iterator iterator() {
	// Returns an iterator over the elements in this collection.  
		return new BalkingQueueIterator(this);
	}

	public boolean remove(Object obj) {
	// Removes a single instance of the specified element from this collection,
	// if it is present (optional operation). 
		downdateHashCode(obj);
		return list.remove(obj);
	}
	 	 
	public boolean removeAll(Collection c) {
	// Removes all this collection's elements that are also contained in the
	// specified collection (optional operation). 
		boolean b = false;
		Iterator ci = c.iterator();
		while (ci.hasNext()) {
			b = remove(ci.next()) || b;
		}
		return b;
	}

	public int size() {
	// Returns the number of elements in this collection.  
		return list.size();
	}
	
	public String toString() {
		String s = super.toString();
		return( "Balking" + s.substring(4, s.length()) );
	}
	
	public static void main(String[] args) {
		BalkingQueue q1 = new BalkingQueue();
		String name;
		
		q1.enqueue("Alice");
		q1.add("Bob");
		q1.enqueue("Carol");
		System.out.println(q1);
		System.out.println("q1.hashCode: " + q1.hashCode());
		System.out.println("Empty? " + q1.isEmpty());
		 
		System.out.println("\nq1 Iterator test:");
		Iterator iter = q1.iterator();
		while (iter.hasNext()) {
			name = (String)iter.next();
			System.out.println(name);
		}
		
		System.out.println("\nLooking at three queues");
		BalkingQueue q2 = new BalkingQueue();
		q2.addAll(q1);
		System.out.println("\nq2 Iterator test:");
		iter = q2.iterator();
		while (iter.hasNext()) {
			name = (String)iter.next();
			System.out.println(name);
		}
		BalkingQueue q3 = new BalkingQueue();
		q3.addAll(q2);
		System.out.println("\nq3 Iterator test:");
		iter = q3.iterator();
		while (iter.hasNext()) {
			name = (String)iter.next();
			System.out.println(name);
		}

		System.out.println("q1 equals q2? " + q1.equals(q2));
		System.out.println("q1 equals q3? " + q1.equals(q3));
		System.out.println("q1.hashCode: " + q1.hashCode());
		System.out.println("q2.hashCode: " + q2.hashCode());
		System.out.println("q3.hashCode: " + q3.hashCode());
		System.out.println("Is Bob in q1? " + q1.contains("Bob"));
		System.out.println("q1.remove(" + "Bob)");
		q1.remove("Bob");
		q1.add("Doug");
		System.out.println("q1: " + q1);
		System.out.println("q1.peek(): " + q1.peek());
		System.out.println("q2.remove(" + "Bob)");
		q2.remove("Bob");
		System.out.println("q1.hashCode: " + q1.hashCode());
		System.out.println("q2.hashCode: " + q2.hashCode());
		System.out.println("q3.hashCode: " + q3.hashCode());
		System.out.println("q1.size: " + q1.size());
		System.out.println("q2.size: " + q2.size());
		System.out.println("q3.size: " + q3.size());
		System.out.println("q1 equals q2? " + q1.equals(q2));
		System.out.println("q1 equals q3? " + q1.equals(q3));
		System.out.println("Is q1 contained in q2? " + q2.containsAll(q1));
		System.out.println("Is q2 contained in q1? " + q1.containsAll(q2));
		System.out.println("Is q2 contained in q3? " + q3.containsAll(q3));
		
		System.out.println("\nLooking at cloning");
		BalkingQueue q4 = (BalkingQueue)q1.clone();
		System.out.println("q1 equals q4? " + q1.equals(q4));
		System.out.println("q1.hashCode: " + q1.hashCode());
		System.out.println("q4.hashCode: " + q4.hashCode());
		System.out.println("q4.dequeue: " + q4.dequeue());
		System.out.println("q1 equals q4? " + q1.equals(q4));
		System.out.println("q1.hashCode: " + q1.hashCode());
		System.out.println("q4.hashCode: " + q4.hashCode());

		System.out.println("\nLooking at an array and a queue");
		Object[] names = q1.toArray();
		for (int i=0; i<names.length; i++) {
			System.out.println(names[i]);
		}
		String[] moreNames = (String[])q1.toArray(new String[5]);
		System.out.println("moreNames.length: " + moreNames.length);
		for (int i=0; i<moreNames.length; i++) {
			System.out.println(moreNames[i]);
		}

		System.out.println("\nEmptying the queues");
		System.out.println(q1.dequeue());
		System.out.println(q1.hashCode());
		System.out.println(q1.dequeue());
		System.out.println(q1.dequeue());
		System.out.println(q1.dequeue());
		System.out.println("q1 Empty? " + q1.isEmpty());
		q2.clear();
		System.out.println("q2.clear()");
		System.out.println("q2 Empty? " + q2.isEmpty());
		System.out.println("q1 equals q2? " + q1.equals(q2));
		System.out.println("q1.hashCode: " + q1.hashCode());
		System.out.println("q2.hashCode: " + q2.hashCode());
		iter = q3.iterator();
		while (iter.hasNext()) {
			System.out.println("q3 iter.next(): " + iter.next());
			System.out.println("q3 iter.remove()");
			iter.remove();
		}
		System.out.println("q3 Empty? " + q3.isEmpty());
		System.out.println("q3.hashCode: " + q3.hashCode());
		System.out.println("q3.size: " + q3.size());

		System.out.println("\nTesting the iterator for consistency checking");
		iter = q4.iterator();
		try {
			System.out.println(iter.next());
			System.out.println("q4.dequeue(): " + q4.dequeue());
			System.out.println(iter.next());
		} catch (ConcurrentModificationException e) {
			System.out.println("Concurrent Modification Exception: " + e.getMessage());
		}		
	}
}	
