import java.util.*;

public class DWvQueue extends M865Queue {
   public static final int initCap = 100;
   protected Vector q;
   
   public DWvQueue(int capacity, int increment) {
      q = new Vector(capacity, increment);
   }
   
   public DWvQueue(int capacity) {
      q = new Vector(capacity);
   }
   
   public DWvQueue() {
      this(initCap);
   }
   
   //  Append an object onto the queue.
   public void enqueue(Object o) {
      this.add(o);
   }
   
   //  Remove and return the object at the beginning of the queue.
   //  Return null if the queue is empty.
   public Object dequeue() {
      if (q.isEmpty()) {
         return null;
      } else {
         return q.remove(0);
      }
   }
   
   //  Return true if the queue is empty, false otherwise.
   public boolean isEmpty() {
      return q.isEmpty();
   }
   
   //  Return the number of items in the queue.
   public int size() {
      return q.size();
   }
   
   //  Return the object at the beginning of the queue.
   public Object peek() {
      if (q.isEmpty()) {
         return null;
      } else {
         return q.get(0);
      }
   }
   
   //  Methods for the Collection Interface.
   public Iterator iterator() {
      return q.iterator();
   }

   //  Use an inner class for the iterator
   public class DWvQIterator implements Iterator {
      private int current;
      
      public DWvQIterator() {
         current = 0;
      }
      public boolean hasNext() {
         return (current < q.size());
      }
      public Object next() {
         return q.get(current++);
      }
      public void remove() {
         current--;
         q.remove(current);
      }
   }

   public Iterator getIterator() {
      return new DWvQIterator();
   }
   
   public void clear() {
      q.clear();
   }

   public boolean add(Object o) {
      //  The behavior is identical to enqueue
      return q.add(o);
   }

   public boolean addAll(Collection c) {
      return q.addAll(c);
   }
   
   public boolean retainAll(Collection c) {
      return q.retainAll(c);   
   }

   public boolean contains(Object o) {
      return q.contains(o);
   }

   public boolean remove(Object o) {
      return q.remove(o);
   }

   public Object[] toArray() {
      return q.toArray();
   }

   public Object[] toArray(Object[] a) {
      return q.toArray(a);
   }

   public boolean removeAll(Collection c) {
      return q.removeAll(c);
   }

   public boolean containsAll(Collection c) {
      return q.containsAll(c);
   }
     
        
   //  Simple debugging routine
    public static void main(String args[]) {
       Integer myData;
       DWvQueue myQueue = new DWvQueue();
       
       System.out.println("Queue empty: " + myQueue.isEmpty());
       System.out.println("Queue size: " + myQueue.size());
       for (int k=0; k<5; k++) {
          myData = new Integer(k);
          System.out.print("Append: " + myData + "    ");
          myQueue.enqueue(myData);
          System.out.println("Peek: " + (Integer)myQueue.peek());
       }
       System.out.println("Test the iterator");
       Iterator i = myQueue.iterator();
       while ( i.hasNext() ) {
          System.out.println("i: " + (Integer)i.next());
       }
       System.out.println("Test two iterators");
       Iterator j;
       i = myQueue.iterator();
       while ( i.hasNext() ) {
          System.out.print("i: " + (Integer)i.next() + "   j:");
          j = myQueue.iterator();
          while ( j.hasNext() ) {
             System.out.print(" " + (Integer)j.next());
          }
          System.out.println();
       }
       System.out.println("Test DWvQIterator");
       i = myQueue.getIterator();
       while ( i.hasNext() ) {
          System.out.println("i: " + (Integer)i.next());
       }
       System.out.println("Test two DWvQIterators");
       i = myQueue.getIterator();
       while ( i.hasNext() ) {
          System.out.print("i: " + (Integer)i.next() + "   j:");
          j = myQueue.getIterator();
          while ( j.hasNext() ) {
             System.out.print(" " + (Integer)j.next());
          }
          System.out.println();
       }
       System.out.println("Queue empty: " + myQueue.isEmpty());
       System.out.println("Queue size: " + myQueue.size());
       while ( !myQueue.isEmpty() ) {
          System.out.println("Dequeue: " + (Integer)myQueue.dequeue());
       }
       System.out.println("Queue empty: " + myQueue.isEmpty());
       System.out.println("Queue size: " + myQueue.size());
       myData = (Integer)myQueue.dequeue();
       System.out.println("Dequeue: " + myData);


       System.out.println("Test remove with iterator");
       System.out.println("Queue empty: " + myQueue.isEmpty());
       for (int k=0; k<5; k++) {
          myData = new Integer(k);
          System.out.print("Append: " + myData + "    ");
          myQueue.enqueue(myData);
          System.out.println("Peek: " + (Integer)myQueue.peek());
       }
       System.out.println("Remove with DWvQIterators");
       i = myQueue.getIterator();
       while ( i.hasNext() ) {
          myData = (Integer)i.next();
          System.out.print("i: " + myData + "   j:");
          if (myData.intValue() == 2) {
             i.remove();
          }
          j = myQueue.getIterator();
          while ( j.hasNext() ) {
             myData = (Integer)j.next();
             System.out.print(" " + myData);
             if (myData.intValue() == 0  ||  myData.intValue() == 3) {
                j.remove();
             }
          }
          System.out.println();
       }
       
       System.out.println("Test remove with a single builtin iterator");
       myQueue.clear();
       System.out.println("Queue empty: " + myQueue.isEmpty());
       for (int k=0; k<5; k++) {
          myData = new Integer(k);
          System.out.print("Append: " + myData + "    ");
          myQueue.enqueue(myData);
          System.out.println("Peek: " + (Integer)myQueue.peek());
       }
       System.out.println("Remove with the iterator");
       i = myQueue.iterator();
       while ( i.hasNext() ) {
          myData = (Integer)i.next();
          System.out.print("i: " + myData + "   j:");
          if (myData.intValue() == 2) {
             i.remove();
          }
          j = myQueue.iterator();
          while ( j.hasNext() ) {
             myData = (Integer)j.next();
             System.out.print(" " + myData);
          }
          System.out.println();
       }
       
       System.out.println("Test remove with two builtin iterators");
       myQueue.clear();
       System.out.println("Queue empty: " + myQueue.isEmpty());
       for (int k=0; k<5; k++) {
          myData = new Integer(k);
          System.out.print("Append: " + myData + "    ");
          myQueue.enqueue(myData);
          System.out.println("Peek: " + (Integer)myQueue.peek());
       }
       System.out.println("Remove with the iterator");
       i = myQueue.iterator();
       while ( i.hasNext() ) {
          myData = (Integer)i.next();
          System.out.print("i: " + myData + "   j:");
          if (myData.intValue() == 2) {
             i.remove();
          }
          j = myQueue.iterator();
          while ( j.hasNext() ) {
             myData = (Integer)j.next();
             System.out.print(" " + myData);
             if (myData.intValue() == 0  ||  myData.intValue() == 3) {
                j.remove();
             }
          }
          System.out.println();
       }
   }
   
}
