import java.util.*;

public class QueueBenchmark {
   public static final int MAXSIZE = 100000;
   public static final int NLOOPS = 10;
   public static int[] qLength = {100, 200, 500, 1000, 2000, 5000, 10000, 20000, 50000, 100000};
   public static int nQLengths = 10;

   public static void main(String args[]) {

      //  Initialize the data objects to be used in the tests
      Integer[] myData = new Integer[MAXSIZE];       
      for (int k=0; k<MAXSIZE; k++) {
         myData[k] = new Integer(k);
         if (10*k % MAXSIZE == 0) {
            System.out.println("myData[ " + k + "] = " + myData[k]);          
         }
      }

      //  Simple debugging check of the five key methods of an M865Queue
      M865Queue aQueue;
//***
      System.out.println("***  Creating a DWvQueue.");
      aQueue = new DWvQueue();
//***
      System.out.println("Is empty ?" + aQueue.isEmpty());
      System.out.println("Queue size: " + aQueue.size());
      System.out.println("Front of queue: " + aQueue.peek());
      System.out.println("Pushining 10 items.");
      for (int k=0; k<10; k++) {
         aQueue.enqueue(myData[k]);
         System.out.println("Appending: " + myData[k]);

      }
      System.out.println("***  Checking the iterator and removing element 5.");
      System.out.println("Front of queue: " + aQueue.peek());
      System.out.println("Is empty? " + aQueue.isEmpty());
      System.out.println("Queue size: " + aQueue.size());
      Iterator i = aQueue.iterator();
      Integer val;
      while (i.hasNext()) {
         val = (Integer)i.next();
         System.out.println("Looking at: " + val);
         if (val.intValue() == 5) {
            i.remove();
            System.out.println("Removed: " + val);
         }
      }
      System.out.println("***  Poping all items.");
      System.out.println("Front of queue: " + aQueue.peek());
      System.out.println("Is empty ?" + aQueue.isEmpty());
      System.out.println("Queue size: " + aQueue.size());
      while (!aQueue.isEmpty()) {
         val = (Integer)aQueue.dequeue();
         System.out.println("Popped: " + val);
      }
      System.out.println("Front of queue: " + aQueue.peek());
      System.out.println("Is empty ?" + aQueue.isEmpty());
      System.out.println("Queue size: " + aQueue.size());         
      val = (Integer)aQueue.dequeue();
      System.out.println("Popped: " + val);
      
      // Begin the benchmarking
      double [][] results = new double[nQLengths][5];
      int j,k,kk;
      long maxLoops;
      double startTime, stopTime;
      
      for (kk=0; kk<nQLengths; kk++) {
         results[kk][0] = qLength[kk];
         maxLoops = NLOOPS*MAXSIZE/qLength[kk];
         
         // Timings for creating a queue.
         startTime = System.currentTimeMillis();
         for (j=0; j<maxLoops; j++) {
//***
            aQueue = new DWvQueue();
//***
         }
         stopTime = System.currentTimeMillis();
         results[kk][1] = (stopTime - startTime)/maxLoops;

         // Timings for creating a queue and doing the empty loop.
         startTime = System.currentTimeMillis();
         for (j=0; j<maxLoops; j++) {
//***
            aQueue = new DWvQueue();
//***
            for (k=0; k<qLength[kk]; k++) {
               // Do nothing.
            }
         }
         stopTime = System.currentTimeMillis();
         results[kk][2] = (stopTime - startTime)/maxLoops;

         // Timings for stuffing the queue.
         startTime = System.currentTimeMillis();
         for (j=0; j<maxLoops; j++) {
//***
            aQueue = new DWvQueue();
//***
            for (k=0; k<qLength[kk]; k++) {
               aQueue.enqueue(myData[k]);
            }
         }
         stopTime = System.currentTimeMillis();
         results[kk][3] = (stopTime - startTime)/maxLoops;
 
         // Timings for stuffing and emptying the queue.
         startTime = System.currentTimeMillis();
         for (j=0; j<maxLoops; j++) {
//***
            aQueue = new DWvQueue();
//***
            for (k=0; k<qLength[kk]; k++) {
               aQueue.enqueue(myData[k]);
            }
            k = 0;
            while (!aQueue.isEmpty()) {
               val = (Integer)aQueue.dequeue();
               if (!(val==myData[k++])) {
                  System.out.println("*** Problem *** val: " + val + "   myData: " + myData[--k]);
                  break;
               }
            }
         }
         stopTime = System.currentTimeMillis();
         results[kk][4] = (stopTime - startTime)/maxLoops;
         
         // Report the results
         System.out.print(" " + results[kk][0]);
         System.out.print("  " + results[kk][1]);
         System.out.print("  " + results[kk][2]);
         System.out.print("  " + results[kk][3]);
         System.out.println("  " + results[kk][4]);
      }      
   }
}
