/*
    Keyword Search Application
    
    The objective is to build a program which reads from the
    standard input. The first line will be a comma-separated
    list of keywords. The program is to read the remaining
    lines of input, identify every line that contains one
    or more of the keywords, and indicate which keywords are
    in that line.

    More specifically.
    1. Read the first line and build a queue containing the
       keywords.
    2. Build a Finite State Machine which will recognize the
       keywords.
       a. Determine all forward transitions and their states.
          Do this in a breadth-first fashion, so that the
          succeeeding step is simplified.
       b. Build the failure function and complete the finite
          state transition function.
       c. Build an appropriate action function.
    3. Read the remainder of the file character by character
       and process each character with your Finite State Machine.
    4. For each line, if the line contains one or more keywords,
       print the line number followed by a colon and a space, and
       then print the list of keywords in that line.

    Treat upper case and lower case characters as the same character.
    Assume that there will be no more than 40 keywords and that
    keywords will only contain the letters a thru z.

    I will send you one or more test files to run thru your program.
    You should turn in (1) a short discussion of your design and
    implementation, (2) a printout of your program, and (3) a
    printout of the test results.

*/
import java.io.*;

public class KeywordRecognizer {

    public static void main(String args[]) {
        KeywordTokenizer reader = new KeywordTokenizer(System.in);
        
        int line=0, token=0;
        while (token != KeywordTokenizer.KW_EOF) {
            try {
                token = reader.nextKeyword();
                if (token != KeywordTokenizer.KW_EOF) {
                    if (line != reader.lineno) {
                        line = reader.lineno;
                        System.out.print("\n" + Integer.toString(line) + ": " + reader.keyword );
                    } else {
                         System.out.print(", " + reader.keyword );
                    }
                }
            } catch (IOException e) {} 
        }
        System.out.print("\n");
    }            
}
        
