import java.util.*;

public class PriorityQueue {
    protected Vector myPQ;
	
    public PriorityQueue() {
        myPQ = new Vector();
    }
	
    public PriorityQueue(int capacity) {
        myPQ = new Vector(capacity);
    }
	
    public PriorityQueue(int capacity, int increment) {
        myPQ = new Vector(capacity, increment);
    }

    public boolean isEmpty() {
        return myPQ.isEmpty();
    }

    public int size() {
        return myPQ.size();
    }
    
    public void enqueue(PQRecord rec) {
        PQRecord pRec;
        int k, p;
        addElement(rec);
        k = size() - 1;
        while (0 < k) {
           //  bubble up.
           p = parentOf(k);
           pRec = elementAt(p);
           if ( PQRecord.higherThan(rec, pRec) ) {
               setElementAt(rec,p);
               setElementAt(pRec,k);
               k = p;
           } else {
               break;
           }
       }   
    }

    public PQRecord dequeue() {
        PQRecord topRec, kRec, hcRec;
        int k, hc, oc;
        if ( isEmpty() ) {
            return null;
        } else {
            topRec = elementAt(0);
            k = size() - 1;
            kRec = elementAt(k);
            removeElementAt(k);
            k = 0;
            if (0 < size()) {
                setElementAt(kRec, k);
                // Sift down
                while ((hc = leftChildOf(k)) < size()) {
                    hcRec = elementAt(hc);
                    if ((oc = rightChildOf(k)) < size()) {
                       if (PQRecord.higherThan(elementAt(oc), hcRec)) {
                           hc = oc;
                           hcRec = elementAt(hc);
                       }
                    }
                    if (PQRecord.higherThan(hcRec, kRec)) {
                       // swap
                       setElementAt(hcRec, k);
                       setElementAt(kRec, hc);
                       k = hc;
                    } else {
                        break;
                    }
                }
            }
            return topRec;
        }
    }
    
    protected void addElement(PQRecord rec) {
        myPQ.addElement( rec );
    }
    
    protected PQRecord elementAt(int k) {
        return (PQRecord)myPQ.elementAt(k);
    }
    
    protected void setElementAt(PQRecord rec, int k) {
        myPQ.setElementAt(rec, k);
    }
    
    protected void removeElementAt(int k) {
        myPQ.removeElementAt(k);
    }
    
    protected int parentOf(int k) {
       return Math.round((k-1.5f)/2);
    }
    
    protected int leftChildOf(int k) {
       return (2*k+1);
    }
    
    protected int rightChildOf(int k) {
       return (2*k+2);
    }
    
    public String toString() {
        return "Priority Queue: " + myPQ.toString();
    }
    
    public static void main(String args[]) {
        PriorityQueue pq = new PriorityQueue();

        pq.enqueue(new PQRecord(new LowIntPriority(5), "Jane"));
        System.out.println( pq );

        pq.enqueue(new PQRecord(new LowIntPriority(7), "Dick"));
        System.out.println( pq );

        pq.enqueue(new PQRecord(new LowIntPriority(6), "Spot"));
        System.out.println( pq );

        pq.enqueue(new PQRecord(new LowIntPriority(3), "Alice"));
        System.out.println( pq );

        while (!pq.isEmpty()) {
            System.out.println(pq.dequeue());
            System.out.println( pq );
        }
    }

}

